﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Messaging;
using GalaSoft.MvvmLight.Threading;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Windows;
using System.Windows.Navigation;
using System;
using Microsoft.Phone.Controls;
using Mnemuse.Model;

namespace Mnemuse.ViewModel
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class CharactersViewModel : ViewModelBase
    {        
        /// <summary>
        /// Initializes a new instance of the CharactersViewModel class.
        /// </summary>

        private Book lastBook;
        private IMnemuseServiceProvider serviceProvider;

        public CharactersViewModel(IMnemuseServiceProvider sp)
        {
            serviceProvider = sp;
            Messenger.Default.Register<PropertyChangedMessage<Book>>(
                this,
                (action) => DispatcherHelper.CheckBeginInvokeOnUI(
                    () => { if (action.NewValue != null) LoadCharactersFromBook(action.NewValue); }));

            if (IsInDesignMode)
            {
                // Code runs in Blend --> create design time data.
                for (int x = 0; x < 10; x++)
                {
                    Character c = new Character { Name = "Character " + (x + 1).ToString() };
                    Characters.Add(c);
                }
            }
            else
            {
                // Code runs "for real": Connect to service, etc...
            }
        }

        public Book LastBook
        {
            get
            {
                return lastBook;
            }
        }

        private void LoadCharactersFromBook(Book b)
        {
            if (b == lastBook) return;

            lastBook = b;

            PhoneApplicationFrame frame = (PhoneApplicationFrame)App.Current.RootVisual;
            if (!frame.CurrentSource.ToString().Contains("BookPivotView.xaml") )
                frame.Navigate(new Uri("/BookPivotView.xaml", UriKind.Relative));
  

            Debug.WriteLine("Loading characters from book '" + b.Title + "'");
            Characters.Clear();
            for (int x = 0; x < 5; x++)
            {
                Character c = new Character { Name = b.Title + " character " + (x + 1).ToString() };
                Characters.Add(c);
            }
        }

        /// <summary>
        /// The <see cref="PageName" /> property's name.
        /// </summary>
        public const string PageNamePropertyName = "PageName";

        private string pageName = "Characters";

        /// <summary>
        /// Gets the PageName property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public string PageName
        {
            get
            {
                return pageName;
            }

            set
            {
                if (pageName == value)
                {
                    return;
                }

                pageName = value;
                
                // Update bindings, no broadcast
                RaisePropertyChanged(PageNamePropertyName);
            }
        }

        /// <summary>
        /// The <see cref="Characters" /> property's name.
        /// </summary>
        public const string CharactersPropertyName = "Characters";

        private ObservableCollection<Character> characters = new ObservableCollection<Character>();

        /// <summary>
        /// Gets the Characters property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public ObservableCollection<Character> Characters
        {
            get
            {
                return characters;
            }

            set
            {
                if (characters == value)
                {
                    return;
                }               
                characters = value;
                
                // Update bindings, no broadcast
                RaisePropertyChanged(CharactersPropertyName);
            }
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean own resources if needed

        ////    base.Cleanup();
        ////}
    }
}