﻿using GalaSoft.MvvmLight;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight.Command;
using System.Windows;
using GalaSoft.MvvmLight.Messaging;
using Mnemuse.Messages;
using System.Threading;
using GalaSoft.MvvmLight.Threading;
using Microsoft.Phone.Controls;
using System;
using Mnemuse.Model;
using System.Collections.Generic;

namespace Mnemuse.ViewModel
{
    /// <summary>
    /// This class contains properties that the main View can data bind to.
    /// <para>
    /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// <para>
    /// You can also use Blend to data bind with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// </summary>
    public class MainViewModel : ViewModelBase
    {
        private ObservableCollection<Book> books;
        private Book selectedBook;
        private IMnemuseServiceProvider serviceProvider;

        public RelayCommand AddBookCommand { get; private set; }         

        #region Singular Properties
        public string ApplicationTitle
        {
            get
            {
                return "Mnemuse";
            }
        }

        public string PageName
        {
            get
            {
                return "Books";
            }
        }

        public string Welcome
        {
            get
            {
                return "Welcome to MVVM Light";
            }
        }

        public Book SelectedBook
        {
            get
            {
                return selectedBook;
            }
            set
            {
                var oldValue = selectedBook;
                if (oldValue == value) return;
                
                selectedBook = value;                
                              
                // broadcast this to all listening viewmodels, not just bound UI
                RaisePropertyChanged("SelectedBook", oldValue, selectedBook, true);

                this.SelectedBook = null; // clear this so user can hit back and re-tap same item
            }
        }
        #endregion

        /// <summary>
        /// The <see cref="IsBusy" /> property's name.
        /// </summary>
        public const string IsBusyPropertyName = "IsBusy";

        private bool isBusy = false;

        /// <summary>
        /// Gets the IsBusy property.
        /// TODO Update documentation:
        /// Changes to that property's value raise the PropertyChanged event. 
        /// This property's value is broadcasted by the Messenger's default instance when it changes.
        /// </summary>
        public bool IsBusy
        {
            get
            {
                return isBusy;
            }

            set
            {
                if (isBusy == value)
                {
                    return;
                }

                var oldValue = isBusy;
                isBusy = value;                

                // Update bindings, no broadcast
                RaisePropertyChanged(IsBusyPropertyName);                
            }
        }

        public ObservableCollection<Book> Books
        {
            get
            {
                return books;
            }
            set
            {
                books = value;
                RaisePropertyChanged("Books");
            }
        }

        /// <summary>
        /// Initializes a new instance of the MainViewModel class.
        /// </summary>
        public MainViewModel(IMnemuseServiceProvider sp)
        {
            serviceProvider = sp;
            Books = new ObservableCollection<Book>();
            
            if (IsInDesignMode)
            {
                IsBusy = false; // toggle this to see the animation in designer
                // Code runs in Blend --> create design time data.
                for (int x = 1; x < 6; x++)
                {
                    Book b = new Book { Title = "Book " + x.ToString(), 
                        Description = "This is book " + x.ToString() };
                    Books.Add(b);
                }
                this.SelectedBook = Books[0];   
            }
            else
            {
                // Code runs "for real"     
                LoadBooks();
                InitializeCommands();
            }

          Messenger.Default.Register<RemoveBookMessage>(this, RemoveBook);
        }

        private void InitializeCommands()
        {
            AddBookCommand = new RelayCommand(
            () => AddBookToList(),
            () => true);            

            AddBookCommand.RaiseCanExecuteChanged();            
        }

        public void RemoveBook(RemoveBookMessage removeMessage)
        {         
            DeleteBookFromList(removeMessage.Book);
        }

        private void AddBookToList()
        {
            Random r = new Random();
            Book b = new Book { Title = "Book " + r.Next(1, 100).ToString(), 
                Description = "This is a new Book",
                ID = Guid.NewGuid()};
            Books.Add(b);
            serviceProvider.SaveBooks(this.Books.ToModelArray());
        }

        private void LoadBooks()
        {
            Books.Clear();
            IEnumerable<Book> books = serviceProvider.LoadBooks().ToViewModel();
            foreach (Book b in books)
                Books.Add(b);
        }

        private void DeleteBookFromList(Book b)
        {                                    
            if (b != null)
                Books.Remove(b);
            serviceProvider.SaveBooks(this.Books.ToModelArray());
        }

        ////public override void Cleanup()
        ////{
        ////    // Clean up if needed

        ////    base.Cleanup();
        ////}
    }
}